@@ -12,24 +12,27 @@ require 'jsonpath/parser'
 # into a token array.
 class JsonPath
   PATH_ALL = '$..*'
+  MAX_NESTING_ALLOWED = 100
 
   DEFAULT_OPTIONS = {
     :default_path_leaf_to_null => false,
     :symbolize_keys => false,
     :use_symbols => false,
-    :allow_send => true
+    :allow_send => true,
+    :max_nesting => MAX_NESTING_ALLOWED
   }
 
   attr_accessor :path
 
   def initialize(path, opts = {})
     @opts = DEFAULT_OPTIONS.merge(opts)
+    set_max_nesting
     scanner = StringScanner.new(path.strip)
     @path = []
     until scanner.eos?
       if (token = scanner.scan(/\$\B|@\B|\*|\.\./))
         @path << token
-      elsif (token = scanner.scan(/[$@a-zA-Z0-9:{}_-]+/))
+      elsif (token = scanner.scan(/[$@\p{Alnum}:{}_ -]+/))
         @path << "['#{token}']"
       elsif (token = scanner.scan(/'(.*?)'/))
         @path << "[#{token}]"
@@ -44,11 +47,9 @@ class JsonPath
       elsif (token = scanner.scan(/[><=] \d+/))
         @path.last << token
       elsif (token = scanner.scan(/./))
-        begin
-          @path.last << token
-        rescue RuntimeError
-          raise ArgumentError, "character '#{token}' not supported in query"
-        end
+        @path.last << token
+      else
+        raise ArgumentError, "character '#{scanner.peek(1)}' not supported in query"
       end
     end
   end
@@ -79,7 +80,7 @@ class JsonPath
 
   def on(obj_or_str, opts = {})
     a = enum_on(obj_or_str).to_a
-    if opts[:symbolize_keys]
+    if symbolize_keys?(opts)
       a.map! do |e|
         e.each_with_object({}) { |(k, v), memo| memo[k.to_sym] = v; }
       end
@@ -87,12 +88,45 @@ class JsonPath
     a
   end
 
+  def self.fetch_all_path(obj)
+    all_paths = ['$']
+    find_path(obj, '$', all_paths, obj.class == Array)
+    return all_paths
+  end
+
+  def self.find_path(obj, root_key, all_paths, is_array = false)
+    obj.each do |key, value|
+      table_params = { key: key, root_key: root_key}
+      is_loop = value.class == Array || value.class == Hash
+      if is_loop
+        path_exp = construct_path(table_params)
+        all_paths << path_exp
+        find_path(value, path_exp, all_paths, value.class == Array)
+      elsif is_array
+        table_params[:index] = obj.find_index(key)
+        path_exp = construct_path(table_params)
+        find_path(key, path_exp, all_paths, key.class == Array) if key.class == Hash || key.class == Array
+        all_paths << path_exp
+      else
+        all_paths << construct_path(table_params)
+      end
+    end
+  end
+
+  def self.construct_path(table_row)
+    if table_row[:index]
+      return table_row[:root_key] + '['+ table_row[:index].to_s + ']'
+    else
+      return table_row[:root_key] + '.'+ table_row[:key]
+    end
+  end
+
   def first(obj_or_str, *args)
     enum_on(obj_or_str).first(*args)
   end
 
   def enum_on(obj_or_str, mode = nil)
-    JsonPath::Enumerable.new(self, self.class.process_object(obj_or_str), mode,
+    JsonPath::Enumerable.new(self, self.class.process_object(obj_or_str, @opts), mode,
                              @opts)
   end
   alias [] enum_on
@@ -107,11 +141,20 @@ class JsonPath
 
   private
 
-  def self.process_object(obj_or_str)
-    obj_or_str.is_a?(String) ? MultiJson.decode(obj_or_str) : obj_or_str
+  def self.process_object(obj_or_str, opts = {})
+    obj_or_str.is_a?(String) ? MultiJson.decode(obj_or_str, max_nesting: opts[:max_nesting]) : obj_or_str
   end
 
   def deep_clone
     Marshal.load Marshal.dump(self)
   end
+
+  def set_max_nesting
+    return unless @opts[:max_nesting].is_a?(Integer) && @opts[:max_nesting] > MAX_NESTING_ALLOWED
+    @opts[:max_nesting] = false
+  end
+
+  def symbolize_keys?(opts)
+    opts.fetch(:symbolize_keys, @opts&.dig(:symbolize_keys))
+  end
 end
