@@ -70,24 +70,60 @@ class TestJsonpath < MiniTest::Unit::TestCase
     assert_equal [@object['store']['book'][0], @object['store']['book'][2]], JsonPath.new("$..book[?(@['price'] < 10)]").on(@object)
     assert_equal [@object['store']['book'][0], @object['store']['book'][2]], JsonPath.new("$..book[?(@['price'] == 9)]").on(@object)
     assert_equal [@object['store']['book'][3]], JsonPath.new("$..book[?(@['price'] > 20)]").on(@object)
-    assert_equal [
-      @object['store']['book'][0],
-      @object['store']['book'][4],
-      @object['store']['book'][5],
-      @object['store']['book'][6]
-    ], JsonPath.new("$..book[?(@['category'] != 'fiction')]").on(@object)
+  end
+
+  def test_not_equals_operator
+    expected =
+      [
+        @object['store']['book'][0],
+        @object['store']['book'][4],
+        @object['store']['book'][5],
+        @object['store']['book'][6]
+      ]
+    assert_equal(expected, JsonPath.new("$..book[?(@['category'] != 'fiction')]").on(@object))
+    assert_equal(expected, JsonPath.new("$..book[?(@['category']!=fiction)]").on(@object))
+    assert_equal(expected, JsonPath.new("$..book[?(@.category!=fiction)]").on(@object))
+    assert_equal(expected, JsonPath.new("$..book[?(@.category != 'fiction')]").on(@object))
   end
 
   def test_or_operator
     assert_equal [@object['store']['book'][1], @object['store']['book'][3]], JsonPath.new("$..book[?(@['price'] == 13 || @['price'] == 23)]").on(@object)
+    result = ["Sayings of the Century", "Sword of Honour", "Moby Dick", "The Lord of the Rings"]
+    assert_equal result, JsonPath.new("$..book[?(@.price==13 || @.price==9 || @.price==23)].title").on(@object)
+    assert_equal result, JsonPath.new("$..book[?(@.price==9 || @.price==23 || @.price==13)].title").on(@object)
+    assert_equal result, JsonPath.new("$..book[?(@.price==23 || @.price==13 || @.price==9)].title").on(@object)
+  end
+
+  def test_or_operator_with_not_equals
+    # Should be the same regardless of key style ( @.key vs @['key'] )
+    result = ['Nigel Rees', 'Evelyn Waugh', 'Herman Melville', 'J. R. R. Tolkien', 'Lukyanenko']
+    assert_equal result, JsonPath.new("$..book[?(@['title']=='Osennie Vizity' || @['author']!='Lukyanenko')].author").on(@object)
+    assert_equal result, JsonPath.new("$..book[?(@.title=='Osennie Vizity' || @.author != Lukyanenko )].author").on(@object)
+    assert_equal result, JsonPath.new("$..book[?(@.title=='Osennie Vizity' || @.author!=Lukyanenko )].author").on(@object)
   end
 
   def test_and_operator
     assert_equal [], JsonPath.new("$..book[?(@['price'] == 13 && @['price'] == 23)]").on(@object)
+    assert_equal [], JsonPath.new("$..book[?(@.price>=13 && @.category==fiction && @.title==no_match)]").on(@object)
+    assert_equal [], JsonPath.new("$..book[?(@.title==no_match && @.category==fiction && @.price==13)]").on(@object)
+    assert_equal [], JsonPath.new("$..book[?(@.price==13 && @.title==no_match && @.category==fiction)]").on(@object)
+    assert_equal [], JsonPath.new("$..book[?(@.price==13 && @.bad_key_name==true && @.category==fiction)]").on(@object)
+
+    expected = [@object['store']['book'][1]]
+    assert_equal expected, JsonPath.new("$..book[?(@['price'] < 23 && @['price'] > 9)]").on(@object)
+    assert_equal expected, JsonPath.new("$..book[?(@.price < 23 && @.price > 9)]").on(@object)
+
+    expected = ['Sword of Honour', 'The Lord of the Rings']
+    assert_equal expected, JsonPath.new("$..book[?(@.price>=13 && @.category==fiction)].title").on(@object)
+    assert_equal ['The Lord of the Rings'], JsonPath.new("$..book[?(@.category==fiction && @.isbn && @.price>9)].title").on(@object)
+    assert_equal ['Sayings of the Century'], JsonPath.new("$..book[?(@['price'] == 9 && @.author=='Nigel Rees')].title").on(@object)
+    assert_equal ['Sayings of the Century'], JsonPath.new("$..book[?(@['price'] == 9 && @.tags..asdf)].title").on(@object)
   end
 
-  def test_and_operator_with_more_results
-    assert_equal [@object['store']['book'][1]], JsonPath.new("$..book[?(@['price'] < 23 && @['price'] > 9)]").on(@object)
+  def test_and_operator_with_not_equals
+    expected = ['Nigel Rees']
+    assert_equal expected, JsonPath.new("$..book[?(@['price']==9 && @['category']!=fiction)].author").on(@object)
+    assert_equal expected, JsonPath.new("$..book[?(@.price==9 && @.category!=fiction)].author").on(@object)
   end
 
   def test_nested_grouping
@@ -209,7 +245,7 @@ class TestJsonpath < MiniTest::Unit::TestCase
   end
 
   def test_counting
-    assert_equal 57, JsonPath.new('$..*').on(@object).to_a.size
+    assert_equal 59, JsonPath.new('$..*').on(@object).to_a.size
   end
 
   def test_space_in_path
@@ -439,6 +475,16 @@ class TestJsonpath < MiniTest::Unit::TestCase
                  JsonPath.new('$.store._links').on(@object)
   end
 
+  def test_support_for_umlauts_in_member_names
+    assert_equal [@object['store']['Übermorgen']],
+                 JsonPath.new('$.store.Übermorgen').on(@object)
+  end
+
+  def test_support_for_spaces_in_member_name
+    assert_equal [@object['store']['Title Case']],
+                 JsonPath.new('$.store.Title Case').on(@object)
+  end
+
   def test_dig_return_string
     assert_equal ['asdf'], JsonPath.new("$.store.book..tags[?(@ == 'asdf')]").on(@object)
     assert_equal [], JsonPath.new("$.store.book..tags[?(@ == 'not_asdf')]").on(@object)
@@ -570,11 +616,42 @@ class TestJsonpath < MiniTest::Unit::TestCase
         'name' => 'testname2'
       }]
     }
-    assert_equal [{ 'isTrue' => true, 'name' => 'testname1' }], JsonPath.new('$.data[?(@.isTrue)]').on(data)
+
+    # These queries should be equivalent
+    expected = [{ 'isTrue' => true, 'name' => 'testname1' }]
+    assert_equal expected, JsonPath.new('$.data[?(@.isTrue)]').on(data)
+    assert_equal expected, JsonPath.new('$.data[?(@.isTrue==true)]').on(data)
+    assert_equal expected, JsonPath.new('$.data[?(@.isTrue == true)]').on(data)
+
+    # These queries should be equivalent
+    expected = [{ 'isTrue' => false, 'name' => 'testname2' }]
+    assert_equal expected, JsonPath.new('$.data[?(@.isTrue != true)]').on(data)
+    assert_equal expected, JsonPath.new('$.data[?(@.isTrue!=true)]').on(data)
+    assert_equal expected, JsonPath.new('$.data[?(@.isTrue==false)]').on(data)
+  end
+
+  def test_and_operator_with_boolean_parameter_value
+    data = {
+      'data' => [{
+        'hasProperty1' => true,
+        'hasProperty2' => false,
+        'name' => 'testname1'
+      }, {
+        'hasProperty1' => false,
+        'hasProperty2' => true,
+        'name' => 'testname2'
+      }, {
+        'hasProperty1' => true,
+        'hasProperty2' => true,
+        'name' => 'testname3'
+      }]
+    }
+    assert_equal ['testname3'], JsonPath.new('$.data[?(@.hasProperty1 && @.hasProperty2)].name').on(data)
   end
 
   def test_regex_simple
     assert_equal %w[asdf asdf2], JsonPath.new('$.store.book..tags[?(@ =~ /asdf/)]').on(@object)
+    assert_equal %w[asdf asdf2], JsonPath.new('$.store.book..tags[?(@=~/asdf/)]').on(@object)
   end
 
   def test_regex_simple_miss
@@ -818,13 +895,17 @@ class TestJsonpath < MiniTest::Unit::TestCase
     path = "$..book[?((@['author'] == 'Evelyn Waugh' || @['author'] == 'Herman Melville') && (@['price'] == 33 || @['price'] == 9))]"
     assert_equal [@object['store']['book'][2]], JsonPath.new(path).on(@object)
   end
-
-  def test_complex_nested_grouping_unmatched_parent
-    path = "$..book[?((@['author'] == 'Evelyn Waugh' || @['author'] == 'Herman Melville' && (@['price'] == 33 || @['price'] == 9))]"
-    err = assert_raises(ArgumentError, 'should have raised an exception') { JsonPath.new(path).on(@object) }
-    assert_match(/unmatched parenthesis in expression: \(\(false \|\| false && \(false \|\| true\)\)/, err.message)
+  
+  def test_nested_with_unknown_key
+    path = "$..[?(@.price == 9 || @.price == 33)].title"
+    assert_equal ["Sayings of the Century", "Moby Dick", "Sayings of the Century", "Moby Dick"], JsonPath.new(path).on(@object)
   end
 
+  def test_nested_with_unknown_key_filtered_array
+    path = "$..[?(@['price'] == 9 || @['price'] == 33)].title"
+    assert_equal ["Sayings of the Century", "Moby Dick", "Sayings of the Century", "Moby Dick"], JsonPath.new(path).on(@object)
+  end
+  
   def test_runtime_error_frozen_string
     skip('in ruby version below 2.2.0 this error is not raised') if Gem::Version.new(RUBY_VERSION) < Gem::Version.new('2.2.0') || Gem::Version.new(RUBY_VERSION) > Gem::Version::new('2.6')
     json = '
@@ -1098,6 +1179,61 @@ class TestJsonpath < MiniTest::Unit::TestCase
     assert_equal [12, nil, nil], JsonPath.new('$.bars[*].foo', default_path_leaf_to_null: true).on(data)
   end
 
+  def test_raise_max_nesting_error
+    json = {
+      a: {
+        b: {
+          c: {
+          }
+        }
+      }
+    }.to_json
+
+    assert_raises(MultiJson::ParseError) { JsonPath.new('$.a', max_nesting: 1).on(json) }
+  end
+
+  def test_linefeed_in_path_error
+    assert_raises(ArgumentError) { JsonPath.new("$.store\n.book") }
+  end
+
+  def test_with_max_nesting_false
+    json = {
+      a: {
+        b: {
+          c: {
+          }
+        }
+      }
+    }.to_json
+
+    assert_equal [{}], JsonPath.new('$.a.b.c', max_nesting: false).on(json)
+  end
+
+  def test_initialize_with_max_nesting_exceeding_limit
+    json = {
+      a: {
+        b: {
+          c: {
+          }
+        }
+      }
+    }.to_json
+
+    json_obj = JsonPath.new('$.a.b.c', max_nesting: 105)
+    assert_equal [{}], json_obj.on(json)
+    assert_equal false, json_obj.instance_variable_get(:@opts)[:max_nesting]
+  end
+
+  def test_initialize_without_max_nesting_exceeding_limit
+    json_obj = JsonPath.new('$.a.b.c', max_nesting: 90)
+    assert_equal 90, json_obj.instance_variable_get(:@opts)[:max_nesting]
+  end
+
+  def test_initialize_with_max_nesting_false_limit
+    json_obj = JsonPath.new('$.a.b.c', max_nesting: false)
+    assert_equal false, json_obj.instance_variable_get(:@opts)[:max_nesting]
+  end
+
   def example_object
     { 'store' => {
       'book' => [
@@ -1149,7 +1285,37 @@ class TestJsonpath < MiniTest::Unit::TestCase
       },
       '@id' => 'http://example.org/store/42',
       '$meta-data' => 'whatevs',
+      'Übermorgen' => 'The day after tomorrow',
+      'Title Case' => 'A title case string',
       '_links' => { 'self' => {} }
     } }
   end
+
+  def test_fetch_all_path
+    data = {
+      "foo" => nil,
+      "bar" => {
+        "baz" => nil
+      },
+      "bars" => [
+        { "foo" => 12 },
+        { "foo" => nil },
+        { }
+      ]
+    }
+    assert_equal ["$", "$.foo", "$.bar", "$.bar.baz", "$.bars", "$.bars[0].foo", "$.bars[0]", "$.bars[1].foo", "$.bars[1]", "$.bars[2]"], JsonPath.fetch_all_path(data)
+  end
+
+
+  def test_extractore_with_dollar_key
+    json = {"test" => {"$" =>"success", "a" => "123"}}
+    assert_equal ["success"],  JsonPath.on(json, "$.test.$")
+    assert_equal ["123"],  JsonPath.on(json, "$.test.a")
+  end
+
+  def test_symbolize_key
+    data = { "store" => { "book" => [{"category" => "reference"}]}}
+    assert_equal [{"category": "reference"}],  JsonPath.new('$..book[0]', symbolize_keys: true).on(data)
+    assert_equal [{"category": "reference"}],  JsonPath.new('$..book[0]').on(data, symbolize_keys: true)
+  end
 end
